package com.company.testapp.entity;

import io.jmix.core.DeletePolicy;
import io.jmix.core.FileRef;
import io.jmix.core.entity.annotation.JmixGeneratedValue;
import io.jmix.core.entity.annotation.OnDeleteInverse;
import io.jmix.core.metamodel.annotation.InstanceName;
import io.jmix.core.metamodel.annotation.JmixEntity;
import jakarta.persistence.*;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.PositiveOrZero;

import java.util.List;
import java.util.UUID;

@JmixEntity
@Table(name = "GDKM_PRODUCT", indexes = {
        @Index(name = "IDX_GDKM_PRODUCT_CATEGORY", columnList = "CATEGORY_ID")
})
@Entity(name = "gdkm_Product")
public class Product {
    @JmixGeneratedValue
    @Column(name = "ID", nullable = false)
    @Id
    private UUID id;

    @InstanceName
    @Column(name = "PRODUCT_ID", nullable = false)
    @NotNull
    private String productId;

    @Column(name = "NAME", nullable = false)
    @NotNull
    private String name;

    @Column(name = "IS_FEATURED")
    private Boolean isFeatured;

    @Column(name = "IMAGE", length = 1024)
    private FileRef image;

    @Column(name = "ICON", length = 1024)
    private FileRef icon;

    @Column(name = "DESCRIPTION")
    private String description;

    @PositiveOrZero(message = "The Price should be greater than or equal to 0 ")
    @Column(name = "PRICE", nullable = false)
    @NotNull
    private Double price;

    @Column(name = "TYPE_", nullable = false)
    @NotNull
    private String type;

    @Column(name = "CALORIES")
    private Integer calories;

    @Column(name = "AVAILABLE_STOCK")
    private Boolean availableStock;

    @OneToMany(mappedBy = "product")
    private List<AddOnGroup> addOnGroups;

    @OnDeleteInverse(DeletePolicy.CASCADE)
    @JoinColumn(name = "CATEGORY_ID", nullable = false)
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    private Category category;

    public FoodType getType() {
        return type == null ? null : FoodType.fromId(type);
    }

    public void setType(FoodType type) {
        this.type = type == null ? null : type.getId();
    }

    public void setAvailableStock(Boolean availableStock) {
        this.availableStock = availableStock;
    }

    public Boolean getAvailableStock() {
        return availableStock;
    }

    public String getProductId() {
        return productId;
    }

    public void setProductId(String productId) {
        this.productId = productId;
    }

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }

    public Category getCategory() {
        return category;
    }

    public void setCategory(Category category) {
        this.category = category;
    }

    public List<AddOnGroup> getAddOnGroups() {
        return addOnGroups;
    }

    public void setAddOnGroups(List<AddOnGroup> addOnGroups) {
        this.addOnGroups = addOnGroups;
    }

    public void setIcon(FileRef icon) {
        this.icon = icon;
    }

    public FileRef getIcon() {
        return icon;
    }

    public Boolean getFeatured() {
        return isFeatured;
    }

    public void setImage(FileRef image) {
        this.image = image;
    }

    public FileRef getImage() {
        return image;
    }

    public Integer getCalories() {
        return calories;
    }

    public void setCalories(Integer calories) {
        this.calories = calories;
    }

    public Double getPrice() {
        return price;
    }

    public void setPrice(Double price) {
        this.price = price;
    }

    public Boolean getIsFeatured() {
        return isFeatured;
    }

    public void setIsFeatured(Boolean isFeatured) {
        this.isFeatured = isFeatured;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

}