package com.company.testapp.view.category;

import com.company.testapp.entity.Category;
import com.company.testapp.repository.CategoryRepository;
import com.company.testapp.view.main.MainView;
import com.vaadin.flow.component.ClickEvent;
import com.vaadin.flow.component.HasValueAndElement;
import com.vaadin.flow.component.formlayout.FormLayout;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.router.Route;
import io.jmix.core.LoadContext;
import io.jmix.core.SaveContext;
import io.jmix.core.validation.group.UiCrossFieldChecks;
import io.jmix.flowui.component.UiComponentUtils;
import io.jmix.flowui.component.validation.ValidationErrors;
import io.jmix.flowui.kit.action.ActionPerformedEvent;
import io.jmix.flowui.kit.component.button.JmixButton;
import io.jmix.flowui.model.CollectionContainer;
import io.jmix.flowui.model.DataContext;
import io.jmix.flowui.model.InstanceContainer;
import io.jmix.flowui.model.InstanceLoader;
import io.jmix.flowui.view.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Collection;
import java.util.List;
import java.util.Set;

import static io.jmix.core.repository.JmixDataRepositoryUtils.*;

@Route(value = "categories", layout = MainView.class)
@ViewController(id = "gdkm_Category.list")
@ViewDescriptor(path = "category-list-view.xml")
@LookupComponent("categoriesDataGrid")
@DialogMode(width = "64em")
public class CategoryListView extends StandardListView<Category> {

    @Autowired
    private CategoryRepository repository;

    @ViewComponent
    private DataContext dataContext;

    @ViewComponent
    private CollectionContainer<Category> categoriesDc;

    @ViewComponent
    private InstanceContainer<Category> categoryDc;

    @ViewComponent
    private InstanceLoader<Category> categoryDl;

    @ViewComponent
    private VerticalLayout listLayout;

    @ViewComponent
    private FormLayout form;

    @ViewComponent
    private HorizontalLayout detailActions;

    @Subscribe
    public void onBeforeShow(final BeforeShowEvent event) {
        updateControls(false);
    }

    @Subscribe("categoriesDataGrid.create")
    public void onCategoriesDataGridCreate(final ActionPerformedEvent event) {
        dataContext.clear();
        Category entity = dataContext.create(Category.class);
        categoryDc.setItem(entity);
        updateControls(true);
    }

    @Subscribe("categoriesDataGrid.edit")
    public void onCategoriesDataGridEdit(final ActionPerformedEvent event) {
        updateControls(true);
    }

    @Subscribe("saveButton")
    public void onSaveButtonClick(final ClickEvent<JmixButton> event) {
        Category item = categoryDc.getItem();
        ValidationErrors validationErrors = validateView(item);
        if (!validationErrors.isEmpty()) {
            ViewValidation viewValidation = getViewValidation();
            viewValidation.showValidationErrors(validationErrors);
            viewValidation.focusProblemComponent(validationErrors);
            return;
        }
        dataContext.save();
        categoriesDc.replaceItem(item);
        updateControls(false);
    }

    @Subscribe("cancelButton")
    public void onCancelButtonClick(final ClickEvent<JmixButton> event) {
        dataContext.clear();
        categoryDl.load();
        updateControls(false);
    }

    @Subscribe(id = "categoriesDc", target = Target.DATA_CONTAINER)
    public void onCategoriesDcItemChange(final InstanceContainer.ItemChangeEvent<Category> event) {
        Category entity = event.getItem();
        dataContext.clear();
        if (entity != null) {
            categoryDl.setEntityId(entity.getId());
            categoryDl.load();
        } else {
            categoryDl.setEntityId(null);
            categoryDc.setItem(null);
        }
        updateControls(false);
    }

    protected ValidationErrors validateView(Category entity) {
        ViewValidation viewValidation = getViewValidation();
        ValidationErrors validationErrors = viewValidation.validateUiComponents(form);
        if (!validationErrors.isEmpty()) {
            return validationErrors;
        }
        validationErrors.addAll(viewValidation.validateBeanGroup(UiCrossFieldChecks.class, entity));
        return validationErrors;
    }

    private void updateControls(boolean editing) {
        UiComponentUtils.getComponents(form).forEach(component -> {
            if (component instanceof HasValueAndElement<?, ?> field) {
                field.setReadOnly(!editing);
            }
        });

        detailActions.setVisible(editing);
        listLayout.setEnabled(!editing);
    }

    private ViewValidation getViewValidation() {
        return getApplicationContext().getBean(ViewValidation.class);
    }

    @Install(to = "categoriesDl", target = Target.DATA_LOADER)
    private List<Category> listLoadDelegate(LoadContext<Category> context) {
        return repository.findAll(buildPageRequest(context), buildRepositoryContext(context)).getContent();
    }

    @Install(to = "categoriesDataGrid.remove", subject = "delegate")
    private void categoriesDataGridRemoveDelegate(final Collection<Category> collection) {
        repository.deleteAll(collection);
    }

    @Install(to = "categoryDl", target = Target.DATA_LOADER)
    private Category detailLoadDelegate(LoadContext<Category> context) {
        return repository.getById(extractEntityId(context), context.getFetchPlan());
    }

    @Install(target = Target.DATA_CONTEXT)
    private Set<Object> saveDelegate(SaveContext saveContext) {
        return Set.of(repository.save(categoryDc.getItem()));
    }
}