package com.company.testapp.view.product;

import com.company.testapp.entity.Product;
import com.company.testapp.repository.ProductRepository;
import com.company.testapp.view.main.MainView;
import com.vaadin.flow.router.Route;
import io.jmix.core.LoadContext;
import io.jmix.flowui.DialogWindows;
import io.jmix.flowui.ViewNavigators;
import io.jmix.flowui.component.grid.DataGrid;
import io.jmix.flowui.kit.action.ActionPerformedEvent;
import io.jmix.flowui.model.DataContext;
import io.jmix.flowui.view.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Collection;
import java.util.List;
import java.util.Objects;

import static io.jmix.core.repository.JmixDataRepositoryUtils.buildPageRequest;
import static io.jmix.core.repository.JmixDataRepositoryUtils.buildRepositoryContext;

@Route(value = "products", layout = MainView.class)
@ViewController(id = "gdkm_Product.list")
@ViewDescriptor(path = "product-list-view.xml")
@LookupComponent("productsDataGrid")
@DialogMode(width = "64em")
public class ProductListView extends StandardListView<Product> {

    @Autowired
    private ProductRepository repository;
    @Autowired
    private DialogWindows dialogWindows;
    @ViewComponent
    private DataGrid<Product> productsDataGrid;
    @ViewComponent
    private DataContext dataContext;
    @Autowired
    private ViewNavigators viewNavigators;

    @Install(to = "productsDl", target = Target.DATA_LOADER)
    private List<Product> loadDelegate(LoadContext<Product> context) {
        return repository.findAll(buildPageRequest(context), buildRepositoryContext(context)).getContent();
    }

    @Install(to = "productsDataGrid.remove", subject = "delegate")
    private void productsDataGridRemoveDelegate(final Collection<Product> collection) {
        repository.deleteAll(collection);
    }

    @Subscribe("productsDataGrid.create")
    public void onProductsDataGridCreate(final ActionPerformedEvent event) {
        viewNavigators.detailView(this, Product.class)
                .newEntity()
                .navigate();
    }

    @Subscribe("productsDataGrid.edit")
    public void onProductsDataGridEdit(final ActionPerformedEvent event) {
        viewNavigators.detailView(this, Product.class)
                .editEntity(Objects.requireNonNull(productsDataGrid.getSingleSelectedItem()))
                .navigate();
    }
}